@@ -39,44 +39,64 @@ Operation in specific cases:
     tree.
 * To reroll with limited depth, use `Die.reroll()`, or `Again()` with no
     modification.
-* When used with `OutcomeCountEvaluator`, the entire evaluation is rerolled.
+* When used with `MultisetEvaluator`, the entire evaluation is rerolled.
 """
 
 # Expose certain names at top-level.
 
+import icepool.population.func
 from icepool.population.func import (d, __getattr__, coin, one_hot,
                                      cartesian_product,
                                      from_cumulative_quantities, from_rv,
                                      min_outcome, max_outcome, align,
-                                     align_range, reduce, accumulate, apply,
-                                     apply_sorted)
+                                     align_range, reduce, accumulate, apply)
+
+apply_sorted = icepool.population.func.apply_sorted
+"""As apply() but sorts the outcomes first.
+
+This is more efficient than `apply()` but still not particularly efficient.
+
+You can use `apply_sorted[index]()` to only keep certain sorted indexes.
+See `Pool.keep()` for documentation.
+
+Args:
+    func: A function that takes one argument per input `Die` and returns an
+        argument to `Die()`.
+    *args: `func` will be called with all sorted joint outcomes of these.
+"""
 
 from icepool.population.base import Population
 from icepool.population.die import implicit_convert_to_die, Die
 from icepool.population.again import Again
 from icepool.population.die_with_truth import DieWithTruth
 
-from icepool.counts import CountsKeysView, CountsValuesView, CountsItemsView
+from icepool.collections import CountsKeysView, CountsValuesView, CountsItemsView
 
-from icepool.population.lowest_highest import lowest, highest, sum_lowest, sum_highest
+from icepool.population.keep import lowest, highest, middle
 
 from icepool.generator.pool import Pool, standard_pool
-from icepool.generator.outcome_count_generator import OutcomeCountGenerator, NextOutcomeCountGenerator, implicit_convert_to_generator
-from icepool.evaluator.outcome_count_evaluator import OutcomeCountEvaluator
+
+from icepool.generator.multiset_generator import MultisetGenerator, NextMultisetGenerator
+from icepool.evaluator.multiset_evaluator import MultisetEvaluator
 
 from icepool.population.deck import Deck
 from icepool.generator.deal import Deal
 
-from icepool.generator.suit import SuitGenerator
+from icepool.expression.multiset_expression import MultisetExpression, implicit_convert_to_expression
+from icepool.expression.multiset_function import multiset_function
 
+import icepool.expression as expression
+import icepool.generator as generator
 import icepool.evaluator as evaluator
 
+import icepool.typing as typing
+
 __all__ = [
     'd', 'coin', 'one_hot', 'cartesian_product', 'Outcome', 'Die', 'Population',
     'Again', 'CountsKeysView', 'CountsValuesView', 'CountsItemsView',
     'from_cumulative_quantities', 'from_rv', 'align', 'align_range', 'lowest',
-    'highest', 'sum_lowest', 'sum_highest', 'min_outcome', 'max_outcome',
-    'reduce', 'accumulate', 'apply', 'apply_sorted', 'Reroll', 'RerollType',
-    'OutcomeCountGenerator', 'Pool', 'standard_pool', 'OutcomeCountEvaluator',
-    'Order', 'evaluator', 'Deck', 'Deal', 'SuitGenerator'
+    'highest', 'middle', 'min_outcome', 'max_outcome', 'reduce', 'accumulate',
+    'apply', 'apply_sorted', 'Reroll', 'RerollType', 'Pool', 'standard_pool',
+    'MultisetGenerator', 'MultisetExpression', 'MultisetEvaluator', 'Order',
+    'Deck', 'Deal', 'multiset_function', 'typing'
 ]
