@@ -1,28 +1,25 @@
 __docformat__ = 'google'
 
-from icepool.generator.outcome_count_generator import OutcomeCountGenerator
-from icepool.typing import Outcome
+from icepool.generator.multiset_generator import MultisetGenerator
+from icepool.typing import Outcome, T
 
 from functools import cached_property
 
-from typing import Collection, Iterator, Sequence, TypeAlias, TypeVar
+from typing import Collection, Hashable, Iterator, Sequence, TypeAlias
 
 AlignmentGenerator: TypeAlias = Iterator[tuple['Alignment', Sequence[int], int]]
 
-T_co = TypeVar('T_co', bound=Outcome, covariant=True)
-"""Type variable representing the outcome type."""
 
-
-class Alignment(OutcomeCountGenerator[T_co, tuple[()]]):
+class Alignment(MultisetGenerator[T, tuple[()]]):
     """A generator that only outputs 0 counts with weight 1."""
 
-    def __init__(self, outcomes: Collection[T_co]):
+    def __init__(self, outcomes: Collection[T]):
         self._outcomes = tuple(sorted(outcomes))
 
-    def outcomes(self) -> Sequence[T_co]:
+    def outcomes(self) -> Sequence[T]:
         return self._outcomes
 
-    def counts_len(self) -> int:
+    def output_arity(self) -> int:
         return 0
 
     def _is_resolvable(self) -> bool:
@@ -49,14 +46,6 @@ class Alignment(OutcomeCountGenerator[T_co, tuple[()]]):
     def denominator(self) -> int:
         return 0
 
-    def equals(self, other) -> bool:
-        if not isinstance(other, Alignment):
-            return False
-        return self._outcomes == other._outcomes
-
     @cached_property
-    def _hash(self) -> int:
-        return hash(self._outcomes)
-
-    def __hash__(self) -> int:
-        return self._hash
+    def _key_tuple(self) -> tuple[Hashable, ...]:
+        return Alignment, self._outcomes
