@@ -1,20 +1,14 @@
 __docformat__ = 'google'
 
 import icepool
-from icepool.typing import Outcome
+from icepool.typing import Outcome, T, U
 
 from collections import defaultdict
 from functools import cache
 import itertools
 import math
 
-from typing import Any, Callable, Hashable, Iterable, Iterator, Literal, Sequence, TypeAlias, TypeVar, overload
-
-T = TypeVar('T', bound=Outcome)
-"""An outcome type."""
-
-U = TypeVar('U', bound=Outcome)
-"""Another outcome type."""
+from typing import Any, Callable, Final, Hashable, Iterable, Iterator, Literal, Sequence, TypeAlias, overload
 
 
 @cache
@@ -282,7 +276,8 @@ def iter_cartesian_product(
         if isinstance(arg, icepool.Die):
             return arg.items()
         elif isinstance(arg, icepool.Pool):
-            return arg.expand().items()
+            # Expression evaluators are difficult to type.
+            return arg.expand().items()  # type: ignore
         else:
             return [(arg, 1)]
 
@@ -308,7 +303,7 @@ def apply(
 
     * If the problem is easy to solve by considering how many dice rolled each
         outcome, one outcome at a time, try using
-        `icepool.Pool` and `icepool.OutcomeCountEvaluator`.
+        `icepool.Pool` and `icepool.MultisetEvaluator`.
     * If the problem is easy to solve by considering one additional `Die` at a
         time, try using `reduce()` instead.
     * If the order in which the dice are rolled is not important, you can use
@@ -353,65 +348,37 @@ def apply(
                        again_end=again_end)
 
 
-class apply_sorted():
-    """This is really a function implemented as a class.
+class ApplySorted():
+    """Class producing `apply_sorted` functions."""
 
-    See the "constructor" for details.
-    """
-
-    # Not a true constructor.
-    def __new__(  # type: ignore
-        cls,
+    def __call__(
+        self,
         func:
         'Callable[..., T | icepool.Die[T] | icepool.RerollType | icepool.Again]',
-        *dice,
+        *dice: 'Outcome | icepool.Die',
         again_depth: int = 1,
         again_end: 'T | icepool.Die[T] | icepool.RerollType | None' = None
     ) -> 'icepool.Die[T]':
-        """Applies `func(lowest_outcome, next_lowest_outcome...)` for all sorted joint outcomes of the dice.
-
-        Treat this as an ordinary function, not a constructor.
-
-        `apply_sorted()` is more efficient than `apply()` but still not very
-        efficient. Use `OutcomeCountEvaluator` instead if at all possible.
-
-        You can use `apply_sorted[]` to only see outcomes at particular sorted indexes.
-        For example, `apply_sorted[-2:](func, *dice)` would give the two highest
-        outcomes to `func()`. This is more efficient than selecting outcomes inside
-        `func`.
-
-        Args:
-            func: A function that takes one argument per input `Die` and returns an
-                argument to `Die()`.
-            *dice: Any number of dice (or objects convertible to dice).
-                `func` will be called with all sorted joint outcomes of `dice`,
-                with one argument per die. All outcomes must be totally orderable.
-            again_depth: Forwarded to the final die constructor.
-            again_end: Forwarded to the final die constructor.
-
-        Returns:
-            A `Die` constructed from the outputs of `func` and the weight of rolling
-            the corresponding sorted outcomes.
-        """
         if not callable(func):
             raise TypeError(
                 'The first argument must be callable. Did you forget to provide a function?'
             )
         pool = icepool.Pool(dice)
-        return pool.expand().map(func,
-                                 star=True,
-                                 again_depth=again_depth,
-                                 again_end=again_end)
+        # Expression evaluators are difficult to type.
+        return pool.expand().map(  # type: ignore
+            func,
+            star=True,
+            again_depth=again_depth,
+            again_end=again_end)
 
-    def __class_getitem__(cls,
-                          sorted_roll_counts: int | slice | tuple[int, ...],
-                          /) -> Callable[..., 'icepool.Die']:
-        """Implements `[]` syntax for `apply_sorted`."""
+    def __getitem__(
+        self, index: int | slice | tuple[int, ...]
+    ) -> 'Callable[..., icepool.Die[T]]':
 
         def result(
             func:
             'Callable[..., T | icepool.Die[T] | icepool.RerollType | icepool.Again]',
-            *dice,
+            *dice: 'Outcome | icepool.Die',
             again_depth: int = 1,
             again_end: 'T | icepool.Die[T] | icepool.RerollType | None' = None
         ) -> 'icepool.Die[T]':
@@ -419,14 +386,19 @@ class apply_sorted():
                 raise TypeError(
                     'The first argument must be callable. Did you forget to provide a function?'
                 )
-            if isinstance(sorted_roll_counts, int):
-                return icepool.Pool(dice)[sorted_roll_counts].map(
-                    func, again_depth=again_depth, again_end=again_end)
+            if isinstance(index, int):
+                return icepool.Pool(dice)[index].map(func,
+                                                     again_depth=again_depth,
+                                                     again_end=again_end)
             else:
-                return icepool.Pool(dice)[sorted_roll_counts].expand().map(
+                # Expression evaluators are difficult to type.
+                return icepool.Pool(dice)[index].expand().map(  # type: ignore
                     func,
                     star=True,
                     again_depth=again_depth,
                     again_end=again_end)
 
         return result
+
+
+apply_sorted: Final = ApplySorted()
